"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const apollo_server_express_1 = require("apollo-server-express");
const express_1 = __importDefault(require("express"));
const typeorm_1 = require("typeorm");
const cors_1 = __importDefault(require("cors"));
const logger_1 = __importDefault(require("../logger"));
const loaders_1 = require("../loaders");
const database_connector_1 = __importDefault(require("../connectors/database-connector"));
const rest_loader_1 = __importDefault(require("../loaders/rest.loader"));
const auth_directive_1 = __importDefault(require("./auth.directive"));
const yaml_parser_1 = __importDefault(require("../tool/yaml-parser"));
const tool_1 = require("../tool");
class App {
    constructor() {
        this.gqlLoader = new loaders_1.GqlLoader();
        this.expressApp = express_1.default();
    }
    static playgroundUrl(port) {
        const { ENDPOINT } = process.env;
        return ENDPOINT ? `${ENDPOINT}/graphql` : `http://localhost:${port || 3000}/graphql`;
    }
    async loadConfig(database) {
        const config = await yaml_parser_1.default.parse(tool_1.PathsResolverTool.YAML);
        if (database) {
            config.localConfiguration.database = database;
        }
        else {
            config.localConfiguration.database.synchronize = true;
        }
        this.config = config;
    }
    async buildServer(initContextStorage) {
        try {
            const connection = await this.createDbConnectionAndMigrate();
            this.expressApp.use(cors_1.default());
            await this.buildGraphql(connection);
            const endpoints = await this.buildRest(connection);
            this.expressApp.use('/health', (req, res) => res.json());
            if (initContextStorage) {
                initContextStorage(this.expressApp);
            }
            let server;
            await new Promise((resolve) => {
                const { port } = this.config.localConfiguration.application;
                server = this.expressApp.listen(port, () => {
                    logger_1.default.log(`GraphQL is available on: http://localhost:${port}/graphql`, 'Bootstrap');
                    if (endpoints.length) {
                        logger_1.default.log('REST endpoints are available on:', 'Bootstrap');
                        const endpointsFlatten = {};
                        endpoints.forEach((endpoint) => {
                            (endpointsFlatten[endpoint.route] = endpointsFlatten[endpoint.route] || []).push(endpoint.method);
                        });
                        Object.entries(endpointsFlatten).forEach((endpoint) => {
                            logger_1.default.log(`  -> [${endpoint[1].join(', ').toUpperCase()}] http://localhost:${port}/rest${endpoint[0]}`, 'Bootstrap');
                        });
                    }
                    resolve();
                });
            });
            return server.close.bind(server);
        }
        catch (e) {
            throw new Error(`Server failed to start, error: ${e.message}`);
        }
    }
    async buildGraphql(connection) {
        const contextHandler = await this.gqlLoader.getContextHandler();
        const server = new apollo_server_express_1.ApolloServer({
            playground: {
                endpoint: App.playgroundUrl(this.config.localConfiguration.application.port),
            },
            introspection: true,
            typeDefs: await this.gqlLoader.getTypeDefs(),
            resolvers: await this.gqlLoader.getResolvers(),
            uploads: false,
            schemaDirectives: {
                auth: auth_directive_1.default,
            },
            context: async (expressContext) => {
                const context = {
                    db: { connection },
                    request: expressContext.req,
                };
                if (contextHandler && expressContext.req.body.operationName !== 'IntrospectionQuery') {
                    const result = await contextHandler(context);
                    if (result && typeof result === 'object') {
                        return Object.assign(Object.assign({}, context), result);
                    }
                }
                return context;
            },
        });
        server.applyMiddleware({ app: this.expressApp });
    }
    async buildRest(connection) {
        return new rest_loader_1.default(this.expressApp, { db: { connection } }, this.config.rest).loadEndpoints();
    }
    async createDbConnectionAndMigrate() {
        var _a;
        const { database } = this.config.localConfiguration;
        if ((_a = this.config.serviceConfiguration) === null || _a === void 0 ? void 0 : _a.skipDatabase) {
            logger_1.default.log('Skipping connection to database, to disable - remove skipDatabase flag from codestore.yaml', 'Database');
            return undefined;
        }
        let connection;
        try {
            connection = typeorm_1.getConnectionManager().get('default');
        }
        catch (error) {
            if (error.name === 'ConnectionNotFoundError') {
                logger_1.default.log('Connecting to database', 'Database');
                connection = await database_connector_1.default.createConnection(database);
                logger_1.default.log('Successfully connected', 'Database');
            }
            else
                throw error;
        }
        if (!database.synchronize) {
            await connection.runMigrations();
            logger_1.default.log('Migrations ran', 'Database');
        }
        return connection;
    }
}
exports.default = App;
//# sourceMappingURL=app.js.map